#!/usr/bin/perl
#
#	(c) FRISK Software International
#	author: Svavar Ingi Hermannsson 2003
#
#	http://www.f-prot.com
#
#	This is an installation script for F - Prot Antivirus for Unix.
use strict;
use Cwd;
eval 'use warnings';

my $manDirectory 	= "";
my $fileCheckStatus	= 1;
my $creationMask	= 0755;
my $binDirectory 	= "";
my $sbinDirectory 	= "";
my $man8Directory 	= "";
my $man1Directory 	= "";
my $sourceDirectory 	= "/usr/local/f-prot/";
my $currentDirectory	= "";
my $readLine		= "";
## Posible distributions:
##
##	rh:		RedHat
##	suse: 		SuSE
##	deb: 		Debian
##	md: 		Mandrake
##	undefined: 	(Unknown)
my $distro		= "";

## fprotVersion: 
##
## 	- Workstation
## 	- File Server
## 	- Mail Server
my $fprotVersion 	= 'workstation';

sub detectFprotVersion
{
	if (-e $sourceDirectory . "f-protd")
	{
		$fprotVersion = 'File Server';
	}
	if (-e $sourceDirectory . "tools/scan-mail.pl")
	{
		$fprotVersion = 'Mail Server';
	}
}

## This is a sub function which explains why unzip is needed, if 
## the checkForUnzip sub function was unable to locate it.
sub noticeUnzip
{
	print "Warning: Unable to locate unzip. This program is required\n"
		. "\tin order to be able to run the check-updates.pl script\n"
		. "\tto check for virus signature updates.\n\n";
}

## This is a sub function which tries to locate unzip.
sub checkForUnzip
{
	open (UNZIPCHECK, "which unzip|");
	while ($readLine = <UNZIPCHECK>)
	{
		if ($readLine != "")
		{
			if ($readLine =~ /no unzip/i || $readLine =~ /:/)
			{
				&noticeUnzip;
				return;
			}
		}
	}
}

sub getBinSymbolicLink
{
	my $newBin;
	my $answer;
        $binDirectory = "/bin/";
        if (-e "/usr/local/bin/")
        {
                $binDirectory = "/usr/local/bin/";
        }

        print "Where do you want a symbolic link for F - Prot Antivirus to be generated?\n[" . $binDirectory . "] : ";
        while (1)
	{
		chop($newBin = <STDIN>);
		if ($newBin eq "" && -e $binDirectory)
		{
			return;
		}
		elsif ($newBin eq "")
		{
			print "You must provide a location for the symbolic link location.\n[" . $binDirectory . "] : ";
        	}
		else
		{
			## Here you make sure the user passed a trailing slash/ for the directory
			if (substr($newBin, length($newBin) - 1, 1) ne "/")
			{
				$newBin = $newBin . "/";
			}
			## If the supplied directory exists, you are done.
			if (-e $newBin)
			{
        	        	$binDirectory = $newBin;
				return;
			}

			## The supplied directory does not exist.
			## Do you wish to create it ?
			print "The directory: '" . $newBin . "' does not exist. Do you want to create it? (y/N): ";
			chop($answer = <STDIN>);
			if ($answer =~ /y/i)
			{
				mkdir ($newBin, $creationMask) || 
					die "Error: Unable to create directory: $newBin\nError: $!\n\n"
						. "Exiting...\n";
				$binDirectory = $newBin;
				return;
			}
			else
			{
				print "You must provide a location for the symbolic link location.\n[" 
					. $binDirectory . "] : ";
			}
		}
	}
}

sub getMan8SymbolicLink
{
	my $newMan8 	= "";
	my $answer 	= "";
	## Here you ask for the location of the system administration manuals (man8).
	if (-e "/usr/local/man/man8")
	{
		$man8Directory = "/usr/local/man/man8/";
	}
	elsif (-e "/usr/share/man/man8")
	{
		$man8Directory = "/usr/share/man/man8/";
	}

	print "Where do you want the symbolic links for section 8 manuals to be generated (man8)?\n"
		. "[" . $man8Directory . "] : ";
	while (1)
	{
		chop($newMan8 = <STDIN>);
		if ($newMan8 eq "" && -e $man8Directory)
		{
			return;
		}
		elsif ($newMan8 eq "")
		{
			print "You need to supply a valid entry for the location of symbolic links for your manual pages.\n"
				. "[" . $man8Directory . "] : ";
		}
		else
		{
			if (substr($newMan8, length($newMan8) - 1, 1) ne "/")
			{
				$newMan8 = $newMan8 . "/";
			}

			## If the supplied directory exists, you are done.
			if ( -e $newMan8)
			{
				$man8Directory = $newMan8;
				return;
			}

			## The supplied directory does not exist.
			## Do you want to create it?
			print "The directory: '" . $newMan8 . "' does not exist. Do you want to create it? (y/N): ";
			chop($answer = <STDIN>);
			if ($answer =~ /y/i)
			{
				mkdir ($newMan8, $creationMask) ||
					die "Error: Unable to create directory: $newMan8\nError: $!\n\n"
						. "Exiting...\n";
				$man8Directory = $newMan8;
				return;
			}
			else
			{
				print "You need to supply a valid entry for the location of symbolic links for \n"
					. "your manual pages.\n"
				. "[" . $man8Directory . "] : ";
			}
		}
	}
}

sub getMan1SymbolicLink
{
	my $newMan1 	= "";
	my $answer 	= "";
        ## Here you ask for the location of the executable programs manuals (man1).
        if (-e "/usr/local/man/man1")
        {
                $man1Directory = "/usr/local/man/man1/";
        }
        elsif (-e "/usr/share/man/man1")
        {
                $man1Directory = "/usr/share/man/man1/";
        }

        print "Where do you want the symbolic links for the manuals for executable programs to be generated (man1)?\n"
		. "[" . $man1Directory . "] : ";
	while (1)
	{
		chop($newMan1 = <STDIN>);
		if ($newMan1 eq "" && -e $man1Directory)
		{
			return;
		}
		elsif($newMan1 eq "")
		{
			print "You need to supply a valid entry for the location of symbolic links for your manual pages.\n"
				. "[" . $man1Directory . "] : ";
		}
		else
		{	
			if (substr($newMan1, length($newMan1) - 1, 1) ne "/")
			{
				$newMan1 = $newMan1 . "/";
			} 

			if (-e $newMan1)
			{
				$man1Directory = $newMan1;
				return;
			}

			## The supplied directory does not exist.
			## Do you want to create it?
			print "The directory: '" . $newMan1 . "' does not exist. Do you want to create it? (y/N): ";
			chop($answer = <STDIN>);
			if ($answer =~ /y/)
			{
				mkdir($newMan1, $creationMask) ||
					die "Error: Unable to create directory: $newMan1\nError: $!\n\n"
						. "Exiting...\n";
				$man1Directory = $newMan1;
				return;
			}
			else
			{
				print "You need to supply a valid entry for the location of symbolic links for\n"
					. "your manual pages.\n"
				. "[" . $man1Directory . "] : ";
			}
		}
	}
}

sub getSbinSymbolicLink
{
	my $answer;
	my $newSbin;
	$sbinDirectory  = "/sbin/";

	if (-e "/usr/local/sbin/")
	{
		$sbinDirectory = "/usr/local/sbin/";
	}
	print "Where do you want a symbolic link for the F - Prot Antivirus Daemon Scanner (f-protd) to be generated?\n"
		. "[" . $sbinDirectory . "] : ";
	while (1)
	{
		chop($newSbin = <STDIN>);
		if ($newSbin eq "" && -e $sbinDirectory)
		{
			return;
		}
		elsif($newSbin eq "")
		{
			print "You need to supply a valid entry for the symbolic link to be placed for f-protd .\n"
				. "[" . $sbinDirectory . "] : ";
			return;
		}
		else
		{
			if (substr($newSbin, length($newSbin) - 1, 1) ne "/")
			{
				$newSbin = $newSbin . "/";
			}

			if (-e $newSbin)
			{
				$sbinDirectory = $newSbin;
				return;
			}

			## The supplied directory does not exist.
			## Do you wish to create it?
			print "The directory: '" . $newSbin . "' does not exist. Do you want to create it? (y/N): ";
			chop ($answer = <STDIN>);
			if ($answer =~ /y/i)
			{
				mkdir($newSbin, $creationMask) ||
					die "Error: Unable to create directory: $newSbin\nError: $!\n\n"
						. "Exiting...\n";
				$sbinDirectory = $newSbin;
				return;
			}
			else
			{
				print "You need to supply a valid entry for the f-protd symbolic link to be placed\n"
					. "[" . $sbinDirectory . "] : "; 
			}

		}
	}
}

## You will only accept /usr/local/f-prot as the source directory.
sub getSourceLocation
{
	my $newSource = getcwd() . "/";
	if ($newSource ne $sourceDirectory)
        {
		die 	"This installation script for F - Prot Antivirus only supports installation from:\n"
			. "\t" . $sourceDirectory . "\n\n"
			. "Please decompress the F - Prot Antivirus $fprotVersion under /usr/local,\n"
			. "so that once extracted it will become:\n\n"
			. "\t/usr/local/f-prot. \n(This is particularly useful since all our documentation,\n"
			. "manuals and support services assume this location.)\n\n"
			. "If you want to install f-prot into another directory, you will need to do it manually.\n"
			. "(for further information on manual installation, read the INSTALL.version document)\n"
			. "\nPlease decompress the f-prot " . $fprotVersion . " distribution under /usr/local/\n"
			. "and try re-running this script again from within the created directory:\n"
			. "\t/usr/local/f-prot\n\nExiting...\n";
        }
}

sub getDistribution
{
	if (-e "/etc/redhat-release")
	{
		## This is a Red Hat distribution.
		$distro = 'rh';
		return;
	}

	if (-e "/etc/SuSE-release")
	{
		## This is a SuSE distribution.
		$distro = 'suse';
		return;
	}

	if (-e "/etc/mandrake-release")
	{
		## This is a Mandrake distribution.
		$distro = 'md';
		return;
	}
	
	if (-e "/etc/debian_version")
	{
		## This is a Debian distribution.
		$distro = 'deb';
		return;
	}
	
	## If you were unable to discover which distribution you have..
	$distro = 'undefined';
	return;
}

sub startingConfiguration
{
	print 	"\n######################################################\n"
		. "# You are starting the configuration part of the:    #\n"
		. "#     F - Prot Antivirus installation.               #\n"
		. "######################################################\n\n";
}

sub configureInstallation
{
	&startingConfiguration;
	&getSourceLocation;
	&getBinSymbolicLink;
	&getMan8SymbolicLink;
	&getMan1SymbolicLink;

	## If you have either the File Server or Mail Server version of 
	## F - Prot Antivirus, you'll need to set up the daemon as well. 
	## (If not you've done all you need to do.)
	if ($fprotVersion =~ /workstation/i)
	{
		return;
	}
	&getSbinSymbolicLink;
	&getDistribution;
}

sub checkForPreExistingFoldersAndLinks
{
	if ( -e "/usr/local/f-prot")
	{
		print "/usr/local/f-prot/ already exists\n";
	}
}

sub printWelcome
{
	print "(c) FRISK Software International\n\nhttp://www.f-prot.com/\n\n";
	print "\tF - Prot Antivirus - $fprotVersion - for Unix.\n\n"
	
}

## This sub function takes three arguments:
##	$sourceFile
##	$destDir
##	$destFile
##
## this function should be called like the following:
##
## &linkFile("sourceFile", "destinationdirectory/", "destinationFile");
sub linkFile
{
	my $sourceFile	= $_[0];
	my $destDir 	= $_[1];
	my $destFile 	= $_[2];
	my $answer	= "";

	print "Setting up symbolic link: " . $destDir . $destFile . ": ";
	if (-l $destDir . $destFile || -e _)
	{
		if (-l $destDir . $destFile)
		{
			my $preExistingLink = readlink($destDir . $destFile) ||
				die "Unable to read already existing symbolic link: '" . $destDir . $destFile
					. "\nError: " . $! . "\nExiting...\n";
			if ($preExistingLink eq $sourceDirectory . $sourceFile)
			{
				print "\n\t(symbolic link already exists) Continuing...\n";
				return
			}
			else
			{
				## There's no reason to let the user know about this.
				unlink($destDir . $destFile) ||
					die "Unable to remove pre-existing link: '" . $destDir . $destFile
						. "'\nError: " . $! . "\nExiting...\n";
				## If you were able to unlink without problems, you'll continue down
				## to the symlink bellow.
			}
		}
		elsif (-d $destDir . $destFile)
		{
			print "A directory already exists with the same name. Do you want to remove it (Y/n): ";
			chop($answer = <STDIN>);
			if ($answer =~ /n/i)
			{
				## If you don't want to remove the directory.
				die "A directory already exists where you wanted to place a symbolic link:\n"
					. "\t" . $destDir . $destFile . "/\n"
					. "Remove the directory or choose a different location for your binary symbolic link"
					. "\nExiting...\n";
			}

			## You wish to remove the directory. (Here you assume the directory is empty)
			rmdir($destDir . $destFile) ||
				die "Unable to remove directory: " . $destDir . $destFile
					. "/\nError: " . $! . "\nExiting...\n";

			## If you were able to remove the directory you'll continue.
		}
		else
		{
			print $destDir . $destFile . " already exists.\nDo you want to remove it? (y/N): ";
			chop($answer = <STDIN>);
			if ($answer =~ /y/i)
			{
				unlink($destDir . $destFile) ||
					die "Unable to remove file: '" . $destDir . $destFile
						. "\nError: " . $! . "\nExiting...\n";
			}
			else
			{
				&unSupportedEvent;
			}
		}
	}

	## No symbolic link exists so you can create it.
	symlink($sourceDirectory . $sourceFile, $destDir . $destFile) || 
		die "Unable to create symbolic link: '" . $destDir . $destFile . "\nError: '" . $! . "\nExiting...\n";
	print "OK.\n";
}

sub unSupportedEvent
{
	die "An event has occurred which this installation script does not support.\n"
		. "Please consult the manual installation instructions in INSTALL.version for\n"
		. "information on how to install F - Prot manually, or try re-running this script.\n"
		. "Exiting...\n";
}

sub makeFprotExecutable
{
	print "\n\nChanging file permissions:\n\n";
	chmod(0755, $sourceDirectory . "f-prot") || 
		die "Unable to change file permissions of " . $sourceDirectory . "f-prot\n"
			. "Error: " . $! . "\nExiting...\n";
	print "File permissions: " . $sourceDirectory . "f-prot rwxr-x-r-x\tOK.\n";

	chmod(0700, $sourceDirectory . "tools/check-updates.pl") ||
		die "Unable to change file permissions of " . $sourceDirectory . "tools/check-updates.pl\n"
			. "Error: " . $! . "\nExiting...\n";
	print "File permissions: " . $sourceDirectory . "tools/check-updates.pl rwx------\tOK.\n";

	if ($fprotVersion =~ /workstation/i)
	{
		return;
	}

	chmod(0755, $sourceDirectory . "f-protd") ||
		die "Unable to change file permissions of " . $sourceDirectory . "f-protd\n"
			. "Error: " . $! . "\nExiting...\n";
	print "File permissions: " . $sourceDirectory . "f-protd rwxr-xr-x\tOK.\n";
	
	if ($fprotVersion =~ /file server/i)
	{
		return;
	}

	chmod(0755, $sourceDirectory . "tools/scan-mail.pl") ||
		die "Unable to change file permissions of " . $sourceDirectory . "tools/scan-mail.pl\n"
			. "Error: " . $! . "\nExiting...\n";
	print "File permissions: " . $sourceDirectory . "tools/scan-mail.pl rwxr-xr-x\tOK.\n";

	## You're done.
	return;
}

sub copyRCFile
{
	@_;
	my $sourceFile = $_[0];
	my $destinationFile = $_[1];
	open(SOURCE, $sourceFile) || die "Unable to open source RC file: '" . $sourceFile . "'\n"
		. "Error: " . $! . "\nExiting...\n";
	open(DESTINATION, ">$destinationFile") || die "Unable to open destination RC file: '" . $destinationFile . "'\n"
		. "Error: " . $! . "\nExiting...\n";

	while (<SOURCE>)
	{
		print DESTINATION $_;
	}
	close (SOURCE);
	close (DESTINATION);
	chmod(0755, $destinationFile) ||
		die "Unable to change file permissions of: '" . $destinationFile . "'\n"
			. "Error: " . $! . "\nExiting...\n";
}

sub askStartFprotdNow
{
	my $answer = "";
	print "Do you want to activate the daemon automatically now and everytime your system is rebooted?\n"
		. "(Y/n): ";
	chop($answer = <STDIN>);
	if ($answer =~ /n/i)
	{
		return 0;
	}

	return 1;
}

sub linkRCFiles
{
	if ($distro =~ /rh/i)
	{
		&copyRCFile($sourceDirectory . "tools/rc_scripts/f-protd.rc-redhat", "/etc/rc.d/init.d/f-protd");
		if ( &askStartFprotdNow )
		{
			system("chkconfig --add f-protd");
			system("/etc/rc.d/init.d/f-protd start");
		}
	}
	elsif ($distro =~ /suse/i)
	{
		&copyRCFile($sourceDirectory . "tools/rc_scripts/f-protd.rc-suse", "/etc/rc.d/f-protd");
		if ( &askStartFprotdNow )
		{
			system("chkconfig f-protd on");
			system("/sbin/SuSEconfig");
			system("/etc/rc.d/f-protd start");
		}
	}
	elsif ($distro =~ /deb/i)
	{
		&copyRCFile($sourceDirectory . "tools/rc_scripts/f-protd.rc-debian", "/etc/init.d/f-protd");
		if ( &askStartFprotdNow)
		{
			system("/etc/init.d/f-protd start");
			print "You need to create links from the runlevel directories (f.ex. /etc/rc3.d)\n"
				. "to /etc/init.d/f-protd\n";
		}
	}
	elsif ($distro =~ /md/i)
	{
		&copyRCFile($sourceDirectory . "tools/rc_scripts/f-protd.rc-mandrake", "/etc/rc.d/init.d/f-protd");
		if ( &askFprotdNow )
		{
			system("chkconfig --add f-protd start");
			system("/etc/rc.d/init.d/f-protd start");
		}
	}
	else
	{
		print "You will need to modify the rc-files manually to make them work with your distribution\n"
			. "Consult the INSTALL.version for further information.\n";
	}
}

sub startingInstallation
{
	print 	"\n######################################################\n"
		. "# You are starting the installation part of the:     #\n"
		. "#     F - Prot Antivirus installation.               #\n"
		. "######################################################\n\n";
}

sub performInstallation
{
	&startingInstallation;
	&linkFile("f-prot.sh", $binDirectory, "f-prot");
	&linkFile("man_pages/check-updates.pl.8", $man8Directory, "check-updates.pl.8");
	&linkFile("man_pages/f-prot.1", $man1Directory, "f-prot.1");
	if ($fprotVersion =~ /workstation/i)
	{
		return;
	}
	&linkFile("man_pages/f-protd.8", $man8Directory, "f-protd.8");
	&linkFile("f-protd", $sbinDirectory, "f-protd");
	if ($distro ne "undefined")
	{
		&linkFile("man_pages/f-prot.so.8", $man8Directory, "f-prot.so.8");
		&linkRCFiles;
	}
	if ($fprotVersion =~ /file server/i)
	{
		return;
	}
	&linkFile("man_pages/scan-mail.pl.8", $man8Directory, "scan-mail.pl.8");
}

sub installCompleted
{
	print 	"\n######################################################\n"
		. "# Installation of F - Prot Antivirus is completed.   #\n"
		. "######################################################\n\n";
}

sub checkingFiles
{
	print 	"\n######################################################\n"
		. "# You are starting the file-check part of the:       #\n"
		. "#     F - Prot Antivirus installation.               #\n"
		. "######################################################\n\n";
}

sub checkFile
{
	my $fileToCheck = $_[0];
	print "Checking file: '" . $sourceDirectory . $fileToCheck . "': ";
	if (! -e $sourceDirectory . $fileToCheck)
	{
		print "\n\tError: file is missing.\n";
		$fileCheckStatus = 0;
	}
	else
	{
		print "OK.\n";
	}
}


sub checkFiles
{
	&checkingFiles;
	&checkFile("f-prot");
	&checkFile("man_pages/f-prot.1");
	&checkFile("man_pages/check-updates.pl.8");
	if ($fprotVersion =~ /workstation/i)
	{
		return;
	}
	&checkFile("f-protd");
	&checkFile("man_pages/f-protd.8");
	if ($distro ne "undefined")
	{
		&checkFile("man_pages/f-prot.so.8");
	}
	if ($fprotVersion =~ /file server/i)
	{
		return;
	}
	&checkFile("man_pages/scan-mail.pl.8");
}
&detectFprotVersion;
&printWelcome;
&configureInstallation;
&checkFiles;
if ($fileCheckStatus == 0)
{
	die "\nSome files needed in the installation were not found.\n"
		. "Fatal error.\nExiting...\n";
}
&performInstallation;
&makeFprotExecutable;
&installCompleted;
